'' =================================================================================================
''
''   File....... db_motors.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 04 JAN 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1     = 31                                                  ' programming / terminal
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28

  ADC_DIO = 10                                                  ' adc pins
  ADC_CLK =  9 
  ADC_CS  =  8
  

con { pst formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  adc : "jm_mcp3208_ez"
 

var

  long  pot[8]


dat { HB25 io pins }

  HB25          long    0, 1, 2, 3
    

pub main | t, ch, p

  adc.start(ADC_CS, ADC_CLK, ADC_DIO)                           ' start adc 

  setup_motors                                                  ' setup HB-25 controls

  t := cnt
  repeat
    longfill(@pot, 0, 8)                                        ' clear
    repeat 4                                                    ' accumulate 4 readings
      repeat ch from 0 to 7
        pot[ch] += adc.read(ch, adc#SE)

    repeat ch from 0 to 7
      pot[ch] >>= 2                                             ' divide by 4 to average

    ' convert to speed/direction (microseconds)
    ' -- see adc.scale and adc.map methods
    ' -- example below

    repeat ch from 0 to 3
      pot[ch] := adc.scale(pot[ch], 1000, 2000)                 ' 0..4095 --> 1000..2000

    repeat ch from 0 to 3                                       ' update motors
      p := HB25[ch]
      if (p => 0)
        pulse_out(p, pot[ch])
    
    waitcnt(t += constant(50 * MS_001))                         ' update ever 50ms
  

pub setup_motors | mtr, p

  repeat mtr from 0 to 3                                        ' loop through 4 motors
    p := HB25[mtr]                                              ' get control pin
    if (p => 0)                                                 ' if used
      repeat                                                    ' wait for HB-25 to be ready
      until (ina[p] == 1)
      low(p)                                                    ' set control pin to output
      pause(5)                                                  ' let HB-25 initialize
      pulse_out(p, 1500)                                        ' stop

  pause(5)                                                      ' hold-off for last motor
      

pub pulse_out(pin, usecs) | mask

'' Creates high-going pulse on pin
'' -- leaves pin in output/low state
'' -- uses ctra
'' -- this is a blocking method

  low(pin)                                                      ' make pin output/low

  ctra := (%00100 << 26) | pin                                  ' use NCO-SE mode
  frqa := 1
  phsa := -(usecs * US_001)                                     ' start the pulse

  waitpeq(|< pin, |< pin, 0)                                    ' ensure pulse started
  waitpne(|< pin, |< pin, 0)                                    ' wait for finish

  ctra := 0                                                     ' disable counter
  

con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}